import os
import platform
import shutil
import subprocess
import sys
import logging
from importlib import import_module, reload

import cuitools as subp

from cszp import colortest, cszp_plugin, cszp_setting, cszp_soccer, cszp_module
from cszp.cszp_module import error_dump


def menu(lang, module, Input):
    logger = logging.getLogger("menu")
    cszp_module.killsoccer()
    inp = ""
    subp.reset()
    while not lang.searchcmd("menu", inp):
        subprocess.check_call("clear", shell=True)
        print("\033[0m\033[38;5;172m")
        v = open("./version")
        cszp_module.figlet("cszp " + v.read())
        v.close()
        print("\n")
        q = lang.question("menu")
        inp = Input.Input(q[0], dot=False, normal=False, word=q[1])
        if not lang.searchcmd("menu", inp):
            logger.warning(lang.lang("ERR:そのようなコマンドはありません。"))
            print("\033[38;5;9m" + lang.lang("ERR:そのようなコマンドはありません。"))
            Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)
    if inp == "exit":
        pass
    elif inp == "colortest":
        logger.info("select colortest")
        colortest.colortest()
        logger.info("printed colortest")
        Input.Input(lang.lang("Enterキーを押して続行..."))
        menu(lang, module, Input)
    elif inp == "setting":
        logger.info("select setting")
        cszp_setting.setting(lang, module, Input)
        logger.info("exited setting")
        menu(lang, module, Input)
    elif inp == "reset":
        logger.info("select reset")
        os.remove("./config/*")
        shutil.rmtree("plugins")
        shutil.rmtree("teams")
        logger.info("reseted")
        Input.Input(lang.lang("\n\nリセットが完了しました。\nEnterキーを押して続行..."), dot=False, textcolor="#fdb100")
        logger.info("exited reset")
        menu(lang, module, Input)
    elif inp == "test":
        logger.info("select test")
        cszp_soccer.setting(lang, testmode=True, module=module, Input_=Input)
        logger.info("exited test")
        menu(lang, module, Input)
    elif inp == "start":
        logger.info("select start")
        cszp_soccer.setting(lang, module=module, Input_=Input)
        logger.info("exited start")
        menu(lang, module, Input)
    elif inp == "rrt":
        logger.info("select rrt")
        cszp_soccer.rrt(lang, module=module, Input_=Input)
        logger.info("exited rrt")
        menu(lang, module, Input)
    elif inp == "lang":
        terminal_size = shutil.get_terminal_size()
        printtext = ["Select Language"]
        select = 0
        logger.info("select lang")
        logger.info("list " + str(lang.lang_list))
        logger.info("length " + str(len(lang.lang_list)))
        logger.info("enable lang name " + lang.enable_lang_name)
        for i in range(len(lang.lang_list)):
            if lang.lang_list[str(i)] == lang.enable_lang:
                select = i
                printtext.append(">> " + os.path.splitext(os.path.basename(lang.lang_list[str(i)]))[0])
            else:
                printtext.append("   " + os.path.splitext(os.path.basename(lang.lang_list[str(i)]))[0])
        printtext.append("")
        k = ""
        while k != "\n":
            lentext = max(map(len, printtext))
            for i in range(len(printtext)):
                if i == 0:
                    print("\033[" + str(int(terminal_size[1] / 2 + i)) + ";" + str(
                        int(terminal_size[0] / 2 - lentext / 2)) + "H┏" + printtext[i].center(lentext, '━') + "┓")
                elif i == len(printtext) - 1:
                    print("\033[" + str(int(terminal_size[1] / 2 + i)) + ";" + str(
                        int(terminal_size[0] / 2 - lentext / 2)) + "H┗" + printtext[i].center(lentext, '━') + "┛")
                else:
                    print("\033[" + str(int(terminal_size[1] / 2 + i)) + ";" + str(
                        int(terminal_size[0] / 2 - lentext / 2)) + "H┃" + printtext[i].center(lentext, ' ') + "┃")
            k = subp.Key()
            if k == "\x1b":
                subp.Key()
                k = subp.Key()
                if k == "B":
                    if select < len(lang.lang_list) - 1:
                        select += 1
                elif k == "A":
                    if select > 0:
                        select -= 1

            printtext = ["Select Language"]
            for i in range(len(lang.lang_list)):
                if i == select:
                    printtext.append(">> " + os.path.splitext(os.path.basename(lang.lang_list[str(i)]))[0])
                else:
                    printtext.append("   " + os.path.splitext(os.path.basename(lang.lang_list[str(i)]))[0])
            printtext.append("")

        langf = open("lang", "w")
        langf.write(str(select))
        langf.close()
        lang = cszp_module.terminal(noenter=True)
        logger.info("reloaded")
        logger.info("select lang name " + lang.enable_lang_name)
        lang.autostart(lang)
        logger.info("autostart executed")
        logger.info("exited lang")
        menu(lang, module, Input)
    elif inp == "loop":
        logger.info("select loop")
        cszp_soccer.setting(lang, loopmode=True, module=module, Input_=Input)
        logger.info("exited loop")
        menu(lang, module, Input)
    elif inp == "server":
        logger.info("select server")
        print(lang.lang("Ctrl+Cで閲覧を終了します。"))
        try:
            logger.info("call", "cd html_logs/ && python3 -m http.server 20000")
            subprocess.check_call("cd html_logs/ && python3 -m http.server 20000", shell=True)
        except KeyboardInterrupt:
            pass
        except subprocess.CalledProcessError as e:
            logger.warning(e)
            Input(lang.lang("Enterキーを押して続行..."))
        logger.info("exited server")
        menu(lang, module, Input)
    elif inp == "plugin":
        logger.info("select plugin")
        tmp = cszp_plugin.plugin(module, lang)
        logger.info("exited plugin")
        if tmp is not None:
            menu(tmp, module, Input)
        else:
            menu(lang, module, Input)
    elif inp == "about":
        logger.info("select about")
        print("\033[0m")
        v = open("./version")
        vd = v.read().splitlines()[0]
        v.close()
        printtext = [
            "CSZP VER " + vd,
            "VERSION:" + vd,
            "",
            sys.version.splitlines()[0],
            sys.version.splitlines()[1],
            "Install Location:" + os.getcwd(),
            "",
            "SYSTEM:" + platform.system() + " " + platform.machine(),
            "PLATFORM:" + platform.platform(),
            "PC-NAME:" + platform.node(),
            "PYTHON-IMPLEMENTATION:" + platform.python_implementation()
        ]
        subp.printlist("about cszp", printtext)
        logger.info("exited about")
        menu(lang, module, Input)
    elif inp == "window":
        logger.info("select window")
        try:
            logger.info("call soccerwindow2")
            subprocess.check_call("soccerwindow2")
        except KeyboardInterrupt:
            pass
        except subprocess.CalledProcessError as e:
            logger.warning(e)
            Input(lang.lang("Enterキーを押して続行..."))
        logger.info("exited window")
        menu(lang, module, Input)
    else:
        logger.info("select other")
        sys.path.append(lang.functo("menu", inp)[0][0])
        plugin = import_module(lang.functo("menu", inp)[1][0])
        reload(plugin)
        logger.info("reload plugin module")
        try:
            logger.info("start plugin mode")
            plugin.plugin(lang, inp)
            logger.info("exited plugin mode")
        except Exception:
            error_dump(lang, "PLUGIN ERROR")
        logger.info("exited other")
        menu(lang, module, Input)



import logging
import os
import sys

sys.path.append(os.getcwd())
os.chdir(os.path.abspath(os.path.dirname(__file__)))

try:
    os.mkdir("logs")
except FileExistsError:
    pass

logging.basicConfig(format='[%(asctime)s] - %(filename)s > %(name)s > %(lineno)d [%(levelname)s]: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S', level=logging.DEBUG, filename="cszp/logs/cszp_log.log", filemode="w")

logging.info("level: DEBUG")
logging.debug("Running in module mode.")


def main():
    logger = logging.getLogger("main")
    print("\033[38;5;2mloading Now...")
    logger.info("loading Now...")
    print("1/7")
    logger.debug("set python path:" + os.getcwd())
    import json
    logger.debug("imported json module")
    print("\033[1A2/7")
    import locale
    logger.debug("imported locale module")
    print("\033[1A3/7")
    import platform
    logger.debug("imported platform module")
    print("\033[1A4/7")
    from cszp.cszp_module import Open, terminal, figlet, error_dump, Input
    logger.debug("imported Open, terminal, figlet, error_dump, Input at cszp.cszp_module")
    print("\033[1A5/7")
    from cszp import cszp_menu
    logger.debug("imported cszp_menu at cszp")
    print("\033[1A6/7")
    import cuitools as subp
    logger.debug("imported cuitools module")
    print("\033[1A7/7")
    subp.reset()
    logger.debug("imported subp module")
    print("\033[1;1H\033[0m\033[38;5;172m")
    logger.info("cszp version " + open("./version").read())
    ver = open("./version")
    figlet("cszp " + ver.read())
    # noinspection PyBroadException
    print("")
    logger.info("python ver:" + platform.python_version())
    logger.info("PYTHON-IMPLEMENTATION:" + platform.python_implementation())
    logger.info("PLATFORM:" + platform.platform())
    logger.info("SYSTEM:" + platform.system() + " " + platform.machine())
    if int(sys.version_info[0]) == 3 and int(sys.version_info[1]) >= 6 and int(sys.version_info[2]) >= 1:
        sys.stdout.write("\033[38;5;10m\033[1m[OK] ")
        print("\033[0m\033[38;5;2mpythonVersion\033[38;5;7m:\033[38;5;6m" + platform.python_version())
    else:
        print("\033[0m\033[38;5;2mpythonVersion\033[38;5;7m:\033[38;5;6m" + platform.python_version())
        print("\033[38;5;9m\033[1m[ERR] \033[0mThis program requires python3.6.1 or higher version.")
        logger.error("This program requires python3.6.1 or higher version.")
        logger.critical("cszp exited.")
        sys.exit()

    if not os.path.isdir("config"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: config")
        logger.info("Create directory: config")
        os.mkdir("config")
    if not os.path.isfile("config/hogo.json"):
        print("\033[38;5;4m[INFO]\033[0mCreate file: config/hogo.json")
        logger.info("Create file: config/hogo.json")
        tmp = open("config/hogo.json","w")
        tmp.write("[]")
        tmp.close()
    if not os.path.isfile("config/plus.txt"):
        print("\033[38;5;4m[INFO]\033[0mCreate file: config/plus.txt")
        logger.info("Create file: config/plus.txt")
        open("config/plus.txt","w").close()
    module = Open()
    if not os.path.isdir("html_logs"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: html_logs")
        logger.info("Create directory: html_logs")
        os.mkdir("html_logs")
    data = module.Open("./config/config.conf")
    path = data.read().splitlines()[0].split(",")[7]
    data.close()
    if not os.path.isdir(path):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: csvdata")
        logger.info("Create directory: csvdata")
        os.mkdir(path)
    if not os.path.isdir("plugins"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: plugins")
        logger.info("Create directory: plugins")
        os.mkdir("plugins")
    if not os.path.isdir("teams"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: teams")
        logger.info("Create directory: teams")
        os.mkdir("teams")

    # main

    if not os.path.isfile("lang"):
        file = open("language/lang.json")
        lang_list = json.load(file)
        file.close()
        langf = open("lang", "w")
        langn = [k for k, n in lang_list.items() if n == locale.getlocale()[0].lower() + ".lang"]
        if len(langn) == 1:
            logger.info("setlang:", langn[0])
            langf.write(langn[0])
        else:
            logger.info("setlang:1")
            langf.write("1")
        langf.close()
    lang = terminal()
    logger.info("inited")
    lang.autostart(lang)
    logger.info("autostart executed")
    Input = Input()
    try:
        logger.info("start menu")
        cszp_menu.menu(lang, module, Input)
    except KeyboardInterrupt:
        pass
    except EOFError:
        pass
    except Exception:
        error_dump(lang, "CSZP PROGRAM ERROR",True)

    #  stop
    subp.reset()
    logger.info("cszp exited.")
    print("\033[38;5;10mgood bye!")


if __name__ == '__main__':
    main()
else:
    print("\033[38;5;4m[OK]\tThank you for using cszp.")
    print("\033[38;5;2m[NOTE]\tRunning in module mode.\n")



import json
import locale
import logging
import os
import shutil
import subprocess
import sys
from io import StringIO

import cuitools
import cuitools as subp
from pyfiglet import Figlet
from importlib import import_module

from prompt_toolkit import PromptSession
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.lexers import Lexer
from prompt_toolkit.completion import WordCompleter

from cszp import cszp_lang
import requests
from tqdm import tqdm


def figlet(text, font="larry3d"):
    print(
        Figlet(
            font=font,
            justify="center",
            width=shutil.get_terminal_size()[0]
        ).renderText(text)
    )


class lexer(Lexer):
    def __init__(self, color):
        self.color = color

    def lex_document(self, document):
        def get_line(lineno):
            return list(map(lambda n: (self.color, n), document.lines[lineno]))

        return get_line


class Input:
    def __init__(self):
        self.session = PromptSession()

    def Input(self, text, normal=False, textcolor="#89e324", dotcolor="#ffffff", usercolor="#75a0d1", dot=True,
              word=None):
        if word is not None:
            word = WordCompleter(word)
        if dot:
            text = [
                (textcolor + " bold" * normal, text),
                (dotcolor, ":")
            ]
        else:
            text = [
                (textcolor + " bold" * normal, text),
            ]
        if word is None:
            inp = self.session.prompt(text, lexer=lexer(usercolor), auto_suggest=AutoSuggestFromHistory())
        else:
            inp = self.session.prompt(text, lexer=lexer(usercolor), completer=word,
                                      auto_suggest=AutoSuggestFromHistory())
        return inp


def killsoccer(rcss=True):
    if rcss:
        try:
            while len(subprocess.check_output("pidof rcssserver", shell=True).decode("utf-8").split()) != 0:
                subprocess.check_output("killall rcssserver", shell=True)
        except subprocess.CalledProcessError:
            pass
    try:
        while len(subprocess.check_output("pidof soccerwindow2-qt4", shell=True).decode("utf-8").split()) != 0:
            subprocess.check_output("killall soccerwindow2-qt4", shell=True)
    except subprocess.CalledProcessError:
        pass


class Open:
    def __init__(self):
        self.writelist = {
            "config": "soccerwindow2start,on,rcglog output,on,rcllog output,on,logfile output," +
                      os.path.expanduser("~") + "/csvdata",
            "setting": "name,command"
        }
        self.hogo = json.load(open("config/hogo.json"))
        with open("config/plus.txt") as f:
            self.plus = f.read()

    def Open(self, file, mode="r"):
        if os.path.isfile(file):
            data = open(file, mode)
        else:
            data = open(file, "w")
            data.write(self.writelist[os.path.splitext(os.path.basename(file))[0]])
            data.close()
            data = open(file, "r")
        if os.path.splitext(os.path.basename(file))[0] == "setting":
            data = StringIO(data.read() + self.plus)
        return data


class terminal(cszp_lang.lang):
    def question(self, key, note="cszp 簡単サッカー実行プログラム", inp=True):
        title = []
        description = []
        cmd = []
        for j in self.files:
            listf = open(j)
            listd = json.load(listf)
            listf.close()
            if key in listd:
                for i in listd[key]:
                    title.append(i["title"])
                for i in listd[key]:
                    description.append(self.lang(i["description"]))
                for i in listd[key]:
                    cmd.append(i["cmd"])

        titlemax = len(max(title, key=lambda n: len(n)))
        title = list(map(lambda n: n.ljust(titlemax), title))
        temp = self.lang(note) + "\n\n"
        for i in range(len(title)):
            temp += title[i] + " " * 4 + description[i] + "\n"
        if inp:
            temp += ">>> "
        return temp, cmd

    def searchcmd(self, key, text):
        hit = 0
        for j in self.files:

            listf = open(j)
            listd = json.load(listf)
            listf.close()
            if key in listd:
                for i in listd[key]:
                    if i["cmd"] == text.split(" ")[0]:
                        hit = 1
        return hit

    def functo(self, key, text):
        func = []
        path = []
        for j in self.files:
            listf = open(j)
            listd = json.load(listf)
            listf.close()
            if key in listd:
                for i in listd[key]:
                    # print(i["cmd"], text, i["cmd"] == text)
                    if i["cmd"] == text:
                        path.append(os.path.dirname(j))
                        func.append(os.path.dirname(j).replace(".", "").replace("/", ".")[1:] + ".__init__")
        return path, func

    def autostart(self, lang):
        plugin_list = self.functo("auto_start", "auto_start")
        for i in plugin_list[0]:
            sys.path.append(i)
        for i in plugin_list[1]:
            plugin = import_module(i)
            plugin.autostart(lang)


def download_files_progress(lang, file_urls):
    for url in file_urls:
        file_url = url
        try:
            res = requests.get(file_url, stream=True)
        except requests.exceptions.ConnectionError:
            cuitools.box("エラー", [lang.lang("この機能を使用するにはネットワーク接続が必要です。"),
                                 lang.lang("以下のサイトににアクセスできるかどうか確認してください。"), "https://raw.githubusercontent.com/",
                                 lang.lang("Enterキーを押して続行...")])
            k = ""
            while k != "\n":
                k = cuitools.Key()
            break
        file_size = int(res.headers.get('content-length', 0))
        pbar = tqdm(total=file_size, unit="B", unit_scale=True)
        with open("/tmp/" + url.split("/")[-1], 'wb') as file:
            for chunk in res.iter_content(chunk_size=1024):
                file.write(chunk)
                pbar.update(len(chunk))
            pbar.close()


def download_file_mem(lang, file_url):
    try:
        return requests.get(file_url).text
    except requests.exceptions.ConnectionError:
        cuitools.box("エラー", [lang.lang("この機能を使用するにはネットワーク接続が必要です。"),
                             lang.lang("以下のサイトににアクセスできるかどうか確認してください。"), "https://raw.githubusercontent.com/",
                             lang.lang("Enterキーを押して続行...")])
        k = ""
        while k != "\n":
            k = cuitools.Key()


def error_dump(lang, ERR_TYPE="CSZP PROGRAM ERROR",e=False):
    logger = logging.getLogger("error_dump")
    import traceback
    logger.error(ERR_TYPE)
    logger.error(traceback.format_exc())
    print("\033[0m")
    with open("logs/cszp_log.log") as f:
        print(f.read())
    print(lang.lang("ログを確認してください")+"\n"+os.getcwd() + "/logs/cszp_log.log\n"+"-"*50)
    if e:
        raise SystemError
    else:
        subp.Key()